// AtariReceiverDlg.cpp : implementation file
//
// Coded by Henryk Cygert
// Gdynia 1999 Poland
// e-mail:hcygert@optix.pl
//

#include "stdafx.h"
#include <io.h>
#include "AtariReceiver.h"
#include "AtariReceiverDlg.h"
#include "About.h"

#include "T2000PL.h"
#include "T2000CZ.h"
#include "Recorder.h"
#include "FileLoader.h"
#include "DekoderXL.h"
#include "DekoderFM2XL.h"


#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


#define OUT0(S)				Info_Out(S)
#define OUT1(S,v1)			Info_Out(S,(void*)v1)
#define OUT2(S,v1,v2)		Info_Out(S,(void*)v1,(void*)v2)
#define OUT3(S,v1,v2,v3)	Info_Out(S,(void*)v1,(void*)v2,(void*)v3)
#define OUT4(S,v1,v2,v3,v4)	Info_Out(S,(void*)v1,(void*)v2,(void*)v3,(void*)v4)

/////////////////////////////////////////////////////////////////////////////
// CAtariReceiverDlg dialog

CAtariReceiverDlg::CAtariReceiverDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CAtariReceiverDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CAtariReceiverDlg)
	m_vChannel = 1;
	m_vDevice = 0;
	m_vUseDst = TRUE;
	m_vSystem = 0;
	//}}AFX_DATA_INIT
	// Note that LoadIcon does not require a subsequent DestroyIcon in Win32
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
	m_StopThread=0;
}

void CAtariReceiverDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CAtariReceiverDlg)
	DDX_Control(pDX, IDC_SYSTEM, m_System);
	DDX_Control(pDX, IDC_CHECK_EXISTING_FILES, m_UseDst);
	DDX_Control(pDX, IDC_STATIC_DESTINATION, m_DstInfo);
	DDX_Control(pDX, IDC_EDIT_LOG, m_Log);
	DDX_Control(pDX, IDC_COMBO_DEVICE, m_Device);
	DDX_Control(pDX, IDC_COMBO_CHANNEL, m_Channel);
	DDX_Control(pDX, IDC_BUTTON_FILE_DESTINATION, m_File_Dst);
	DDX_Control(pDX, IDCANCEL, m_Cancel);
	DDX_Control(pDX, IDC_BUTTON_ABOUT, m_About);
	DDX_Control(pDX, IDC_BUTTON_STOP, m_Stop);
	DDX_Control(pDX, IDC_BUTTON_START, m_Start);
	DDX_CBIndex(pDX, IDC_COMBO_CHANNEL, m_vChannel);
	DDX_CBIndex(pDX, IDC_COMBO_DEVICE, m_vDevice);
	DDX_Check(pDX, IDC_CHECK_EXISTING_FILES, m_vUseDst);
	DDX_CBIndex(pDX, IDC_SYSTEM, m_vSystem);
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CAtariReceiverDlg, CDialog)
	//{{AFX_MSG_MAP(CAtariReceiverDlg)
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	ON_BN_CLICKED(IDCANCEL, OnExit)
	ON_BN_CLICKED(IDC_BUTTON_START, OnButtonStart)
	ON_BN_CLICKED(IDC_BUTTON_STOP, OnButtonStop)
	ON_BN_CLICKED(IDC_BUTTON_FILE_DESTINATION, OnButtonFileDestination)
	ON_BN_CLICKED(IDC_BUTTON_CLEAR_LOG, OnButtonClearLog)
	ON_BN_CLICKED(IDC_BUTTON_ABOUT, OnButtonAbout)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CAtariReceiverDlg message handlers

BOOL CAtariReceiverDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	// Set the icon for this dialog.  The framework does this automatically
	//  when the application's main window is not a dialog
	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon
	
	// TODO: Add extra initialization here
	m_DstPath="";
	m_Log.LimitText();
	m_Mode=WAITING;
	LoadConfig();
	UpdateData(FALSE);
	UpdateMode();
	return TRUE;  // return TRUE  unless you set the focus to a control
}

// If you add a minimize button to your dialog, you will need the code below
//  to draw the icon.  For MFC applications using the document/view model,
//  this is automatically done for you by the framework.

void CAtariReceiverDlg::OnPaint() 
{
	if (IsIconic())
	{
		CPaintDC dc(this); // device context for painting

		SendMessage(WM_ICONERASEBKGND, (WPARAM) dc.GetSafeHdc(), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialog::OnPaint();
	}
}

// The system calls this to obtain the cursor to display while the user drags
//  the minimized window.
HCURSOR CAtariReceiverDlg::OnQueryDragIcon()
{
	return (HCURSOR) m_hIcon;
}

void CAtariReceiverDlg::OnExit() 
{
	// TODO: Add your control notification handler code here
	if(m_Mode == WAITING) {
		StopThread();
		UpdateData();
		SaveConfig();
		
		OnCancel();
	}
}

void CAtariReceiverDlg::UpdateMode()
{
	switch(m_Mode) {
	case DISABLED:
		m_Stop.EnableWindow(FALSE);
		m_About.EnableWindow(FALSE);
		m_Cancel.EnableWindow(FALSE);
		m_File_Dst.EnableWindow(FALSE);
		m_Start.EnableWindow(FALSE);
		m_Channel.EnableWindow(FALSE);
		m_Device.EnableWindow(FALSE);
		m_UseDst.EnableWindow(FALSE);
		m_System.EnableWindow(FALSE);
		break;
	case WAITING:
		m_Stop.EnableWindow(FALSE);
		m_About.EnableWindow(TRUE);
		m_Cancel.EnableWindow(TRUE);
		m_File_Dst.EnableWindow(TRUE);
		m_Start.EnableWindow(TRUE);
		m_Channel.EnableWindow(TRUE);
		m_Device.EnableWindow(TRUE);
		m_UseDst.EnableWindow(TRUE);
		m_System.EnableWindow(TRUE);
		break;
	case RECEIVEING:
		m_Stop.EnableWindow(TRUE);
		m_About.EnableWindow(FALSE);
		m_Cancel.EnableWindow(FALSE);
		m_File_Dst.EnableWindow(FALSE);
		m_Start.EnableWindow(FALSE);
		m_Channel.EnableWindow(FALSE);
		m_Device.EnableWindow(FALSE);
		m_UseDst.EnableWindow(FALSE);
		m_System.EnableWindow(FALSE);
		break;
	}
	CString s;
	s=" Destination - '";
	s+=m_DstPath;
	s+="'";
	m_DstInfo.SetWindowText(s);
}

void CAtariReceiverDlg::OnButtonStart() 
{
	// TODO: Add your control notification handler code here
	m_Mode=DISABLED;
	UpdateMode();
	UpdateData();
	if(StartThread()) {
		OUT0("Started ...");
		m_Mode=RECEIVEING;
	} else {
		OUT0("ERROR starting thread");
		m_Mode=WAITING;
	}
	UpdateMode();
}

void CAtariReceiverDlg::OnButtonStop() 
{
	// TODO: Add your control notification handler code here
	m_Mode=DISABLED;
	UpdateMode();
	StopThread();
}

void CAtariReceiverDlg::Info_Out(LPCTSTR szString, void *Val1, void *Val2, void *Val3, void *Val4)
{
	char Buf[512];
	DWORD sz;
	CTime t = CTime::GetCurrentTime();
	sz=m_Log.GetLineCount();
	sz=m_Log.LineIndex(sz-1);
	m_Log.SetSel(sz,-1,FALSE);
	sz=wsprintf(Buf,"<%02i:%02i:%02i> ",t.GetHour(),t.GetMinute(),t.GetSecond());
	sz+=wsprintf(&Buf[sz],szString,Val1,Val2,Val3,Val4);
	wsprintf(&Buf[sz],"\r\n");
	m_Log.ReplaceSel(Buf);
}

static DWORD WINAPI ThreadProc(LPVOID lpParameter) {
	CAtariReceiverDlg *obj=(CAtariReceiverDlg*)lpParameter;
	CAtariDekoder *m_dek=0;
	CDataInput *m_inp=0;
	CSystem *m_sys;
	BOOL Header;
	DWORD Nr;

	// Selected device
	switch(obj->m_vDevice) {
	case 0:
		m_dek=new CDekoderXL;
		break;
	case 1:
		m_dek=new CDekoderFM2XL;
		break;
	default:
		break;
	}
	
	m_inp = new CRecorder;
//	m_inp = new CFileLoader;

	m_dek->m_Reverse=1;
	switch(obj->m_vSystem) {
	case 0:
		m_sys = new CT2000PL(m_dek,m_inp);
		break;
	case 1:
		m_sys = new CT2000CZ(m_dek,m_inp);
		break;
	}
	if(m_dek && m_inp && m_sys) {
		if(m_sys->Open(obj->m_vChannel == 0 ? DC_LEFT : DC_RIGHT,&obj->m_StopThread,(char*)((LPCTSTR)obj->m_DstPath),obj->m_vUseDst)) {
			Header=TRUE;
			while(1) {
				if(0 != obj->m_StopThread) {
					break;
				}
				if(Header) {
					obj->OUT0("Waiting for the header");
					if(m_sys->ProcessHeader()) {
						obj->OUT1("Found : '%s'",m_sys->FileName());
						Header=FALSE;
					} else {
						if(obj->m_StopThread) break;
						obj->OUT1("ERROR : %s",m_sys->GetLastError());
					}
				} else {
					Nr=m_sys->ProcessData();
					if(Nr) {
						if(Nr == 0xFFFFFFFF) {
							obj->OUT1("    File saved correctly (%i bytes)",m_sys->FileSize());
							Header=TRUE;
						} else {
							obj->OUT2("    Saved block %i%s",m_sys->BlockNr(),((Nr&0x80000000) ? " (existing file used)" : ""));
						}
					} else {
						obj->OUT1("    ERROR : %s",m_sys->GetLastError());
						Header=TRUE;
					}
				}
			}
		} else obj->OUT1("ERROR : %s",m_sys->GetLastError());
		delete m_sys;
		delete m_inp;
		delete m_dek;
	} else obj->OUT0("ERROR : Out of memory");
	obj->OUT0("Stopped ...");
	obj->m_Mode=WAITING;
	obj->UpdateMode();
	obj->m_StopThread=0;
	return 0;
}


BOOL CAtariReceiverDlg::StartThread()
{
	HANDLE hd;
	DWORD dw;
	SECURITY_ATTRIBUTES attr;
	if(m_StopThread == 0) {
		attr.bInheritHandle = TRUE;
		attr.nLength = sizeof(attr);
		attr.lpSecurityDescriptor = 0;
		hd=CreateThread(&attr,0,ThreadProc,this,0,&dw);
		if(hd) {
			::SetThreadPriority(hd,THREAD_PRIORITY_TIME_CRITICAL);
			CloseHandle(hd);
			return TRUE;
		}
	}
	return FALSE;
}

BOOL CAtariReceiverDlg::StopThread()
{
	if(m_StopThread == 0) {
		m_StopThread=1;
	}
	return TRUE;
}

void CAtariReceiverDlg::OnButtonFileDestination() 
{
	// TODO: Add your control notification handler code here
	BROWSEINFO bi;
	LPMALLOC pMalloc; 
	LPITEMIDLIST pidl;
	char szPath[MAX_PATH];

	ZeroMemory(&bi, sizeof(bi));
	bi.hwndOwner = m_hWnd;
	bi.pszDisplayName = "Display Name"; 
	bi.lpszTitle = TEXT("Select folder"); 
	bi.ulFlags = BIF_DONTGOBELOWDOMAIN | BIF_RETURNONLYFSDIRS; 
 
	pidl = SHBrowseForFolder(&bi); 
	if(pidl) {
		SHGetPathFromIDList(pidl, szPath);
		if (SHGetMalloc(&pMalloc) == NOERROR) { 
			pMalloc->Free((void *)pidl); 
		}
		m_DstPath=szPath;
	}
	UpdateMode();
}


#define REG_KEY		"SOFTWARE\\HenrykCygertApps\\AtariTurbo\\1.01"

void CAtariReceiverDlg::SaveConfig()
{
	HKEY key;
	if(ERROR_SUCCESS == RegCreateKey(HKEY_LOCAL_MACHINE,REG_KEY,&key)) {
		RegSetValueEx(key,"Destination",0,REG_SZ,(BYTE*)((LPCTSTR)m_DstPath),m_DstPath.GetLength());
		RegSetValueEx(key,"Device",0,REG_DWORD,(BYTE*)&m_vDevice,4);
		RegSetValueEx(key,"Channel",0,REG_DWORD,(BYTE*)&m_vChannel,4);
		RegSetValueEx(key,"UseExistingFiles",0,REG_DWORD,(BYTE*)&m_vUseDst,4);
		RegSetValueEx(key,"System",0,REG_DWORD,(BYTE*)&m_vSystem,4);
	}
}

void CAtariReceiverDlg::LoadConfig()
{
	HKEY key;
	char Buf[MAX_PATH];
	DWORD Size;
	DWORD Type;
	if(ERROR_SUCCESS == RegOpenKey(HKEY_LOCAL_MACHINE,REG_KEY,&key)) {
		Size=sizeof(Buf);
		RegQueryValueEx(key,"Destination",0,&Type,(BYTE*)Buf,&Size);
		m_DstPath=Buf;
		Size=4; RegQueryValueEx(key,"Device",0,&Type,(BYTE*)&m_vDevice,&Size);
		Size=4;	RegQueryValueEx(key,"Channel",0,&Type,(BYTE*)&m_vChannel,&Size);
		Size=4;	RegQueryValueEx(key,"UseExistingFiles",0,&Type,(BYTE*)&m_vUseDst,&Size);
		Size=4;	RegQueryValueEx(key,"System",0,&Type,(BYTE*)&m_vSystem,&Size);
	}
}

void CAtariReceiverDlg::OnButtonClearLog() 
{
	// TODO: Add your control notification handler code here
	m_Log.SetWindowText("");
}

void CAtariReceiverDlg::OnButtonAbout() 
{
	// TODO: Add your control notification handler code here
	CAbout dlg;
	dlg.DoModal();
}
