// Recorder.cpp: implementation of the CRecorder class.
//
// Coded by Henryk Cygert
// Gdynia 1999 Poland
// e-mail:hcygert@optix.pl
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include <mmsystem.h>
#include "Recorder.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

static void CR_FillError(char *Dst, MMRESULT res) {
	char *info=0;
	switch(res) {
	case MMSYSERR_NOERROR: info="No error."; break;
	case MMSYSERR_ALLOCATED: info="Specified resource is already allocated."; break;
	case MMSYSERR_BADDEVICEID: info="Specified device identifier is out of range."; break;
	case MMSYSERR_NODRIVER: info="No device driver is present."; break;
	case MMSYSERR_NOMEM: info="Unable to allocate or lock memory."; break;
	case WAVERR_BADFORMAT: info="Attempted to open with an unsupported waveform-audio format."; break;
	case MMSYSERR_INVALHANDLE: info="Specified device handle is invalid."; break;
	case WAVERR_STILLPLAYING: info="There are still buffers in the queue."; break;
	case WAVERR_UNPREPARED: info="The buffer pointed to by the pwh parameter hasn't been prepared."; break;
	case MMSYSERR_NODRIVERCB: info="driver does not call DriverCallback"; break;
	
	default:
		ASSERT(0);
		info="Unexpected error.";
	}
	sprintf(Dst,info);
}

static void CALLBACK waveInProc(HWAVEIN hwi,UINT uMsg,DWORD dwInstance,DWORD dwParam1,DWORD dwParam2)
{
	WAVEHDR *hdr;
	MMRESULT res;
	CRecorder *rec;
	switch(uMsg) {
	case WIM_OPEN:
		break;
	case WIM_DATA:
		hdr=(WAVEHDR*)dwParam1;
		if(hdr->dwFlags & WHDR_DONE) {
			rec=((CRecorder *)dwInstance);
			rec->m_ptr=hdr->lpData;
			rec->m_ptrSize=hdr->dwBytesRecorded;
			if(hdr->dwUser == 0) {
				rec->m_clk=clock();
//				if(rec->m_buf1 == hdr) {
//					hdr=(WAVEHDR*)rec->m_buf2;
//				} else {
//					hdr=(WAVEHDR*)rec->m_buf1;
//				}
				res=waveInUnprepareHeader(hwi,hdr,sizeof(WAVEHDR));
				hdr->dwFlags=0;
				res=waveInPrepareHeader(hwi,hdr,sizeof(WAVEHDR));
				if(res == MMSYSERR_NOERROR) {
					res=waveInAddBuffer(hwi,hdr,sizeof(WAVEHDR));
					if(res == MMSYSERR_NOERROR) {
					}
				}
			}
		}
		break;
	case WIM_CLOSE:
		break;
	default:
		ASSERT(0);
	}
}

static WAVEHDR *CR_AllocBuffer(DWORD Size) {
	WAVEHDR *b;
	b=new WAVEHDR;
	if(b) {
		ZeroMemory(b,sizeof(WAVEHDR));
		b->lpData=(LPSTR)GlobalAlloc(GMEM_FIXED,Size);
		if(b->lpData) {
			b->dwBufferLength=Size;
			return b;
		}
		delete b;
	}
	return 0;
}

static void CR_FreeBuffer(WAVEHDR *b) {
	if(b) {
		if(b->lpData) {
			GlobalFree(b->lpData);
		}
		delete b;
	}
}


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CRecorder::CRecorder()
{
	m_hd=0;
	m_buf1=0;
	m_buf2=0;
	m_SmpSize=8820*5;
	m_ptr=0;
	m_ptrSize=0;
}

CRecorder::~CRecorder()
{
	Close();
	CR_FreeBuffer((WAVEHDR *)m_buf1);
	CR_FreeBuffer((WAVEHDR *)m_buf2);
}


BOOL CRecorder::Open(DEK_CHANEL Mode)
{
	HWAVEIN hd;
	MMRESULT res;
	WAVEFORMATEX format;

	m_Mode=Mode;
	if(Close()) {
		m_Msg[0]=0;
		format.wFormatTag=WAVE_FORMAT_PCM;
		format.nChannels=2;
		format.nSamplesPerSec=44100;
		format.wBitsPerSample=8;
		format.nBlockAlign=format.nChannels*(format.wBitsPerSample/8);
		format.nAvgBytesPerSec=format.nSamplesPerSec*format.nBlockAlign;
		format.cbSize=0;

		res=waveInOpen(&hd,WAVE_MAPPER,&format,(DWORD)waveInProc,(DWORD)this,CALLBACK_FUNCTION);
		if(res == MMSYSERR_NOERROR) {
			m_hd=(DWORD)hd;
			if(m_buf1) CR_FreeBuffer((WAVEHDR *)m_buf1);
			if(m_buf2) CR_FreeBuffer((WAVEHDR *)m_buf2);
			m_buf1=CR_AllocBuffer(m_SmpSize);
			m_buf2=CR_AllocBuffer(m_SmpSize);
			m_Buffer=(BYTE *)GlobalAlloc(GMEM_FIXED,m_Mode == DC_STEREO ? m_SmpSize : ((m_SmpSize+1)>>1));
			if(m_buf1 && m_buf2 && m_Buffer) {
				res=waveInPrepareHeader((HWAVEIN)m_hd,(WAVEHDR*)m_buf1,sizeof(WAVEHDR));
				if(res == MMSYSERR_NOERROR) {
					res=waveInPrepareHeader((HWAVEIN)m_hd,(WAVEHDR*)m_buf2,sizeof(WAVEHDR));
					if(res == MMSYSERR_NOERROR) {
						res=waveInAddBuffer((HWAVEIN)m_hd,(WAVEHDR*)m_buf1,sizeof(WAVEHDR));
						if(res == MMSYSERR_NOERROR) {
							res=waveInAddBuffer((HWAVEIN)m_hd,(WAVEHDR*)m_buf2,sizeof(WAVEHDR));
							if(res == MMSYSERR_NOERROR) {
								m_ptr=0;
								m_ptrSize=0;
								res=waveInStart((HWAVEIN)m_hd);
								if(res == MMSYSERR_NOERROR) {
									m_OK=1;
									return TRUE;
								}
							}
						}
					}
				}
			} else res=MMSYSERR_NOMEM;
		}
		CR_FillError(m_Msg,res);
	}
	return FALSE;
}

BOOL CRecorder::Close()
{
	m_Msg[0]=0;
	m_OK=0;
	if(m_Buffer) {
		GlobalFree(m_Buffer);
		m_Buffer=0;
	}
	if(m_hd) {
		MMRESULT res;
		if(m_buf1) ((WAVEHDR*)m_buf1)->dwUser=0x00000001;
		if(m_buf2) ((WAVEHDR*)m_buf2)->dwUser=0x00000001;
		res=waveInReset((HWAVEIN)m_hd);
		if(res == MMSYSERR_NOERROR) {
			if(m_buf1) waveInUnprepareHeader((HWAVEIN)m_hd,(WAVEHDR*)m_buf1,sizeof(WAVEHDR));
			if(m_buf2) waveInUnprepareHeader((HWAVEIN)m_hd,(WAVEHDR*)m_buf2,sizeof(WAVEHDR));
			res=waveInClose((HWAVEIN)m_hd);
			if(res == MMSYSERR_NOERROR) {
				m_hd=0;
				return TRUE;
			}
		}
		CR_FillError(m_Msg,res);
		return FALSE;
	}
	return TRUE;
}

DEK_STATUS CRecorder::ProcessNextBlock()
{
	if(m_OK) {
		BYTE *ptr=(BYTE *)m_ptr;
		DWORD Size=m_ptrSize,sz;
		if(ptr && Size) {
			m_ptr=0;
			m_ptrSize=0;
			if(m_Mode == DC_STEREO) {
				CopyMemory(m_Buffer,ptr,Size);
				m_Size=Size;
			} else {
				if(m_Mode == DC_RIGHT) {
					ptr++;
				}
				Size>>=1;
				m_Size=Size;
				for(sz=0;sz<Size;sz++) {
					m_Buffer[sz]=ptr[sz*2];
				}
			}
			return DS_OK;
		}
		return DS_WAIT;
	}
	return DS_ERROR;
}
