// System.cpp: implementation of the CSystem class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "AtariReceiver.h"
#include "System.h"
#include <io.h>

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CSystem::CSystem(CAtariDekoder *dek, CDataInput *inp)
{
	m_bReadExisting=TRUE;
	szDstDir[0]=0;
	m_abort=0;
	m_dek=dek;
	m_inp=inp;
	lpBuffer=0;
	dwBufSize=0;
	m_OK=TRUE;
	Init();
	m_OK=FALSE;
}

CSystem::~CSystem()
{
	Close();
	if(lpBuffer) {
		GlobalFree(lpBuffer);
		lpBuffer=0;
	}
}

BOOL CSystem::Init()
{
	dwReadedSize=0;
	szFileName[0]=0;
	dwFileSize=0;
	dwBlockNr=0;
	szDstFName[0]=0;
	m_Msg[0]=0;
	if(m_OK == FALSE) {
		sprintf(m_Msg,"CSystem - object not opened");
		return FALSE;
	}
	if(m_dek && m_inp) {
		return TRUE;
	}
	sprintf(m_Msg,"CSystem - fatal error");
	return FALSE;
}

void CSystem::Path_Link(char *Dst, char *Path, char *FName)
{
	int i;
	strcpy(Dst,Path);
	i=strlen(Dst);
	if(i) {
		if(Dst[i-1] != '\\') {
			Dst[i]='\\';
			i++;
		}
	}
	strcpy(&Dst[i],FName);
}

BOOL CSystem::GetFreeFileName(char *Dst, char *Path, char *FName)
{
	Dst[0]=0;
	if(FName && FName[0]) {
		FILE *f;
		char name[MAX_PATH];
		char *ptr,*p;
		int Val;
		strcpy(name,FName);
		while(1) {
			Path_Link(Dst,Path,name);
			f=fopen(Dst,"rb");
			if(f == 0) {
				break;
			}
			fclose(f);
			ptr=name;
			ptr+=strlen(name)-1;
			Val=1;
			p=ptr+1;
			if(ptr[0] == ')') {
				while(p>name) {
					if(p[0] == '(') {
						Val=atoi(p+1);
						Val++;
						break;
					}
					p--;
				}
				if(p == name) p=ptr+1;
			}
			sprintf(p,"(%i)",Val);
		}
		return 1;
	}
	return 0;
}

BOOL CSystem::ReallocBuffer(DWORD Size)
{
	if(lpBuffer) {
		if(Size <= dwBufSize) return TRUE;
		GlobalFree(lpBuffer);
	}
	lpBuffer=(BYTE*)GlobalAlloc(GMEM_FIXED,Size);
	if(lpBuffer) {
		dwBufSize=Size;
		return TRUE;
	}
	return FALSE;
}

BOOL CSystem::Open(DEK_CHANEL Mode, volatile BOOL *bAbort, char *szDstDirectory, BOOL bReadExisting)
{
	if(Close()) {
		if(m_inp && m_dek) {
			if(m_inp->Open(Mode)) {
				m_dek->SetBuffer(0,0);
				if(bAbort) {
					m_bReadExisting=bReadExisting;
					m_abort=bAbort;
					if(szDstDirectory) strncpy(szDstDir,szDstDirectory,sizeof(szDstDir));
					m_OK=TRUE;
					return TRUE;
				}
			} else sprintf(m_Msg,m_inp->GetLastError());
		}
	}
	return FALSE;
}

BOOL CSystem::Close()
{
	BOOL ret=TRUE;
	if(m_inp) {
		ret&=m_inp->Close();
		if(ret == FALSE) {
			sprintf(m_Msg,m_inp->GetLastError());
		}
	}
	m_OK=FALSE;
	return ret;
}

DEK_RESULT CSystem::LoadData()
{
	switch(m_inp->ProcessNextBlock()) {
	case DS_OK:
		m_dek->SetBuffer(m_inp->GetBuffer(),m_inp->GetSize());
		return DEK_OK;
	case DS_END:
		sprintf(m_Msg,"Out of data");
		break;
	case DS_WAIT:
		Sleep(50);
		return DEK_OUT_OF_DATA;
	default:
		Sleep(50);
		sprintf(m_Msg,"Error in input data");
		break;
	};
	return DEK_IO_ERROR;
}

void CSystem::FillFileName(BYTE *FName)
{
	int i,j;
	BYTE c;
	if(m_OK) {
		for(i=9;FName[i] == ' ';i--) {
		}
		for(j=0;j<=i;j++) {
			c=FName[j];
			if(c<' ' || c>'z') c='_';
			szFileName[j]=c;
		}
		szFileName[j]=0;
	}
}

BOOL CSystem::SaveData(BYTE *lpBuf, DWORD Size)
{
	if(szDstFName[0] == 0) {
		if(0 == GetFreeFileName(szDstFName,szDstDir,szFileName)) {
			sprintf(m_Msg,"Can't make full file name '%s'",szFileName);
			return FALSE;
		}
	}
	FILE *f;
	BOOL ret=FALSE;
	if(f=fopen(szDstFName,"ab")) {
		if(0 == fseek(f,dwFileSize,SEEK_SET)) {
			if(Size == fwrite(lpBuf,1,Size,f)) {
				dwFileSize+=Size;
				ret=TRUE;
			} else sprintf(m_Msg,"Can't save data");
		} else sprintf(m_Msg,"Can't move file pointer");
		fclose(f);
	} else sprintf(m_Msg,"Can't open file '%s' for saveing",szDstFName);
	return ret;
}


BOOL CSystem::RenameFile()
{
	if(szDstFName[0] == 0) return TRUE;
	char FName[MAX_PATH];
	char FPath[MAX_PATH];
	sprintf(FName,"%s.ERROR",FileName());
	if(0 == GetFreeFileName(FPath,szDstDir,FName)) {
		sprintf(m_Msg,"Can't make full file name '%s'",FName);
		return FALSE;
	}
	if(0 == rename(szDstFName,FPath)) return TRUE;
	sprintf(m_Msg,"Can't rename file '%s' to '%s'",szDstFName,FPath);
	return FALSE;
}

DWORD CSystem::ReadExisting(BYTE *lpBuf, DWORD Size)
{
	if(m_bReadExisting == FALSE || Abort()) return 0xFFFFFFFF;

	char FilePath[MAX_PATH];
	char *ptr;
	long hd;
	_finddata_t dta;
	FILE *f;
	DWORD fSize;
	DWORD ret=0xFFFFFFFF;

	Path_Link(FilePath,szDstDir,FileName());
	ptr=FilePath+strlen(FilePath);
	sprintf(ptr,".ERROR*");
	hd=_findfirst(FilePath,&dta);
	if(hd != -1) {
		do {
			if(0 == (dta.attrib & _A_SUBDIR)) {
				Path_Link(FilePath,szDstDir,dta.name);
				f=fopen(FilePath,"rb");
				if(f) {
					if(0 == fseek(f,0,SEEK_END)) {
						fSize=ftell(f);
						if(fSize>FileSize()) {
							if(0 == fseek(f,FileSize(),SEEK_SET)) {
								if(Size == fread(lpBuf,1,Size,f)) {
									ret=Size;
								}
							}
						}
					}
					fclose(f);
					if(ret != 0xFFFFFFFF) break;
				}
			}
		} while(0 == _findnext(hd,&dta));
		_findclose(hd);
	}
	return ret;
}
