// T2000CZ.cpp: implementation of the CT2000CZ class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "AtariReceiver.h"
#include "T2000CZ.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CT2000CZ::CT2000CZ(CAtariDekoder *dek, CDataInput *inp) : CSystem(dek,inp)
{
	if(m_dek) m_dek->m_SkipFirstBit=TRUE;
}

CT2000CZ::~CT2000CZ()
{

}


char * CT2000CZ::ProcessHeader()
{
	while(!Abort()) {
		if(FALSE == CSystem::Init()) {
			return FALSE;
		}
		if(ReadBlock(1+10+6)) {
			if(lpBuffer[0] != 0xFF) {
				FillFileName(lpBuffer+1);
				m_blkLoad=(lpBuffer[12]<<8)+lpBuffer[11];
				m_blkLength=(lpBuffer[14]<<8)+lpBuffer[13];
				m_blkRun=(lpBuffer[16]<<8)+lpBuffer[15];
				return szFileName;
			}
			break;
		} else {
			if(dwReadedSize >= 1) {
				if(lpBuffer[0] != 0xFF) break;
			}
		}
	}
	return 0;
}

DWORD CT2000CZ::ProcessData()
{
	BYTE Hdr[6];
	WORD End;
	if(ReadBlock(m_blkLength)) {
		dwBlockNr++;
		End=m_blkLoad+m_blkLength-1;
		Hdr[0]=0xFF;
		Hdr[1]=0xFF;
		Hdr[2]=(BYTE)m_blkLoad;
		Hdr[3]=m_blkLoad>>8;
		Hdr[4]=(BYTE)End;
		Hdr[5]=End>>8;
		if(SaveData(Hdr,6)) {
			if(SaveData(lpBuffer,m_blkLength)) {
				Hdr[0]=(BYTE)736;
				Hdr[1]=736>>8;
				Hdr[2]=(BYTE)737;
				Hdr[3]=737>>8;
				Hdr[4]=(BYTE)m_blkRun;
				Hdr[5]=m_blkRun>>8;
				if(SaveData(lpBuffer,6)) {
					return 0xFFFFFFFF;
				}
			}
		}
	}
	RenameFile();
	return 0;
}

BOOL CT2000CZ::ReadBlock(DWORD Size)
{
	BOOL ret;
	DEK_RESULT res;
	DWORD i;
	BYTE b;
	BYTE suma;

	dwReadedSize=0;
	m_Msg[0]=0;

	if(FALSE == ReallocBuffer(Size)) {
		sprintf(m_Msg,"Out of memory");
		return FALSE;
	}

	ret=FALSE;
	suma=0;
	m_dek->SetNextBlock();
	res=DEK_OUT_OF_DATA;
	for(i=0;i<(Size+1);) {
		if(Abort()) {
			sprintf(m_Msg,"Aborted ...");
			break;
		}
		if(res == DEK_OUT_OF_DATA) {
			res=LoadData();
			if(res == DEK_IO_ERROR) break;
		} else {
 			res=m_dek->GetNextByte(&b);
			if(res == DEK_OK) {
				if(i == Size) {
					if(b != suma) {
						sprintf(m_Msg,"Wrong checksum");
						break;
					} else {
						ret=TRUE;
						break;
					}
				} else {
					lpBuffer[i]=b;
					suma^=b;
				}
				i++;
			} else {
				if(res == DEK_IO_ERROR) {
					if(i > 0) {
						sprintf(m_Msg,"Can't decode byte %i",i);
						break;
					}
					m_dek->SetNextBlock();
					res=DEK_OUT_OF_DATA;
				}
			}
		}
	}
	dwReadedSize=i;
	return ret;
}
