// T2000PL.cpp: implementation of the CT2000PL class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "AtariReceiver.h"
#include "T2000PL.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CT2000PL::CT2000PL(CAtariDekoder *dek, CDataInput *inp) : CSystem(dek,inp)
{
	if(m_dek) m_dek->m_SkipFirstBit=FALSE;
}

CT2000PL::~CT2000PL()
{

}

char * CT2000PL::ProcessHeader()
{
	while(!Abort()) {
		if(FALSE == CSystem::Init()) {
			return FALSE;
		}
		if(ReadBlock(2+10)) {
			if(lpBuffer[0] == 0 && lpBuffer[1] == 0xFF) {
				FillFileName(lpBuffer+2);
				return szFileName;
			}
			break;
		} else {
			if(dwReadedSize >= 2) {
				if(lpBuffer[0] == 0 && lpBuffer[1] == 0xFF) break;
			}
		}
	}
	return 0;
}

DWORD CT2000PL::ProcessData()
{
	DWORD Size;
	BOOL bExisting=FALSE;
	if(0 == ReadBlock(3072+2)) {
		Size=ReadExisting(lpBuffer+2,3072);
		if(Size == 0xFFFFFFFF) {
			RenameFile();
			return 0;
		}
		bExisting=TRUE;
	} else {
		Size=0;
		Size=(lpBuffer[1]<<8)+lpBuffer[0];
	}
	dwBlockNr++;
	if(SaveData(lpBuffer+2,Size)) {
		if(Size == 3072) return bExisting ? dwBlockNr|0x80000000 : dwBlockNr;
		else return 0xFFFFFFFF;
	}
	RenameFile();
	return 0;
}

BOOL CT2000PL::ReadBlock(DWORD Size)
{
	BOOL ret;
	DEK_RESULT res;
	DWORD i;
	BYTE b;
	BYTE suma;

	dwReadedSize=0;
	m_Msg[0]=0;

	if(FALSE == ReallocBuffer(Size)) {
		sprintf(m_Msg,"Out of memory");
		return FALSE;
	}

	ret=FALSE;
	suma=0;
	m_dek->SetNextBlock();
	res=DEK_OUT_OF_DATA;
	for(i=0;i<(Size+1);) {
		if(Abort()) {
			sprintf(m_Msg,"Aborted ...");
			break;
		}
		if(res == DEK_OUT_OF_DATA) {
			res=LoadData();
			if(res == DEK_IO_ERROR) break;
		} else {
 			res=m_dek->GetNextByte(&b);
			if(res == DEK_OK) {
				if(i == Size) {
					if(b != suma) {
						sprintf(m_Msg,"Wrong checksum");
						break;
					} else {
						ret=TRUE;
						break;
					}
				} else {
					lpBuffer[i]=b;
					suma+=b;
				}
				i++;
			} else {
				if(res == DEK_IO_ERROR) {
					if(i > 0) {
						sprintf(m_Msg,"Can't decode byte %i",i);
						break;
					}
					m_dek->SetNextBlock();
					res=DEK_OUT_OF_DATA;
				}
			}
		}
	}
	dwReadedSize=i;
	return ret;
}
