// AtariReceiverDlg.cpp : implementation file
//
// Coded by Henryk Cygert
// Gdynia 1999 Poland
// e-mail:hcygert@optix.pl
//

#include "stdafx.h"
#include <io.h>
#include "AtariReceiver.h"
#include "AtariReceiverDlg.h"
#include "About.h"

#include "DekoderXL.h"
#include "DekoderFM2XL.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


#define OUT0(S)				Info_Out(S)
#define OUT1(S,v1)			Info_Out(S,(void*)v1)
#define OUT2(S,v1,v2)		Info_Out(S,(void*)v1,(void*)v2)
#define OUT3(S,v1,v2,v3)	Info_Out(S,(void*)v1,(void*)v2,(void*)v3)
#define OUT4(S,v1,v2,v3,v4)	Info_Out(S,(void*)v1,(void*)v2,(void*)v3,(void*)v4)

/////////////////////////////////////////////////////////////////////////////
// CAtariReceiverDlg dialog

CAtariReceiverDlg::CAtariReceiverDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CAtariReceiverDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CAtariReceiverDlg)
	m_vChannel = 1;
	m_vDevice = 0;
	m_vUseDst = TRUE;
	//}}AFX_DATA_INIT
	// Note that LoadIcon does not require a subsequent DestroyIcon in Win32
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
	m_StopThread=0;
}

void CAtariReceiverDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CAtariReceiverDlg)
	DDX_Control(pDX, IDC_CHECK_EXISTING_FILES, m_UseDst);
	DDX_Control(pDX, IDC_STATIC_DESTINATION, m_DstInfo);
	DDX_Control(pDX, IDC_EDIT_LOG, m_Log);
	DDX_Control(pDX, IDC_COMBO_DEVICE, m_Device);
	DDX_Control(pDX, IDC_COMBO_CHANNEL, m_Channel);
	DDX_Control(pDX, IDC_BUTTON_FILE_DESTINATION, m_File_Dst);
	DDX_Control(pDX, IDCANCEL, m_Cancel);
	DDX_Control(pDX, IDC_BUTTON_ABOUT, m_About);
	DDX_Control(pDX, IDC_BUTTON_STOP, m_Stop);
	DDX_Control(pDX, IDC_BUTTON_START, m_Start);
	DDX_CBIndex(pDX, IDC_COMBO_CHANNEL, m_vChannel);
	DDX_CBIndex(pDX, IDC_COMBO_DEVICE, m_vDevice);
	DDX_Check(pDX, IDC_CHECK_EXISTING_FILES, m_vUseDst);
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CAtariReceiverDlg, CDialog)
	//{{AFX_MSG_MAP(CAtariReceiverDlg)
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	ON_BN_CLICKED(IDCANCEL, OnExit)
	ON_BN_CLICKED(IDC_BUTTON_START, OnButtonStart)
	ON_BN_CLICKED(IDC_BUTTON_STOP, OnButtonStop)
	ON_BN_CLICKED(IDC_BUTTON_FILE_DESTINATION, OnButtonFileDestination)
	ON_BN_CLICKED(IDC_BUTTON_CLEAR_LOG, OnButtonClearLog)
	ON_BN_CLICKED(IDC_BUTTON_ABOUT, OnButtonAbout)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CAtariReceiverDlg message handlers

BOOL CAtariReceiverDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	// Set the icon for this dialog.  The framework does this automatically
	//  when the application's main window is not a dialog
	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon
	
	// TODO: Add extra initialization here
	m_DstPath="";
	m_Log.LimitText();
	m_Mode=WAITING;
	LoadConfig();
	UpdateData(FALSE);
	UpdateMode();
	return TRUE;  // return TRUE  unless you set the focus to a control
}

// If you add a minimize button to your dialog, you will need the code below
//  to draw the icon.  For MFC applications using the document/view model,
//  this is automatically done for you by the framework.

void CAtariReceiverDlg::OnPaint() 
{
	if (IsIconic())
	{
		CPaintDC dc(this); // device context for painting

		SendMessage(WM_ICONERASEBKGND, (WPARAM) dc.GetSafeHdc(), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialog::OnPaint();
	}
}

// The system calls this to obtain the cursor to display while the user drags
//  the minimized window.
HCURSOR CAtariReceiverDlg::OnQueryDragIcon()
{
	return (HCURSOR) m_hIcon;
}

void CAtariReceiverDlg::OnExit() 
{
	// TODO: Add your control notification handler code here
	if(m_Mode == WAITING) {
		StopThread();
		UpdateData();
		SaveConfig();
		
		OnCancel();
	}
}

void CAtariReceiverDlg::UpdateMode()
{
	switch(m_Mode) {
	case DISABLED:
		m_Stop.EnableWindow(FALSE);
		m_About.EnableWindow(FALSE);
		m_Cancel.EnableWindow(FALSE);
		m_File_Dst.EnableWindow(FALSE);
		m_Start.EnableWindow(FALSE);
		m_Channel.EnableWindow(FALSE);
		m_Device.EnableWindow(FALSE);
		m_UseDst.EnableWindow(FALSE);
		break;
	case WAITING:
		m_Stop.EnableWindow(FALSE);
		m_About.EnableWindow(TRUE);
		m_Cancel.EnableWindow(TRUE);
		m_File_Dst.EnableWindow(TRUE);
		m_Start.EnableWindow(TRUE);
		m_Channel.EnableWindow(TRUE);
		m_Device.EnableWindow(TRUE);
		m_UseDst.EnableWindow(TRUE);
		break;
	case RECEIVEING:
		m_Stop.EnableWindow(TRUE);
		m_About.EnableWindow(FALSE);
		m_Cancel.EnableWindow(FALSE);
		m_File_Dst.EnableWindow(FALSE);
		m_Start.EnableWindow(FALSE);
		m_Channel.EnableWindow(FALSE);
		m_Device.EnableWindow(FALSE);
		m_UseDst.EnableWindow(FALSE);
		break;
	}
	CString s;
	s=" Destination - '";
	s+=m_DstPath;
	s+="'";
	m_DstInfo.SetWindowText(s);
}

void CAtariReceiverDlg::OnButtonStart() 
{
	// TODO: Add your control notification handler code here
	m_Mode=DISABLED;
	UpdateMode();
	UpdateData();
	if(m_rec.Start()) {
		OUT0("Started ...");
		StartThread();
		m_Mode=RECEIVEING;
	} else {
		OUT1("ERROR starting : '%s'",m_rec.GetLastError());
		m_Mode=WAITING;
	}
	UpdateMode();
}

void CAtariReceiverDlg::OnButtonStop() 
{
	// TODO: Add your control notification handler code here
	m_Mode=DISABLED;
	UpdateMode();
	StopThread();
	if(m_rec.Stop()) {
		OUT0("Stopped");
	} else {
		OUT1("ERROR stopping : '%s'",m_rec.GetLastError());
	}
}

void CAtariReceiverDlg::Info_Out(LPCTSTR szString, void *Val1, void *Val2, void *Val3, void *Val4)
{
	char Buf[512];
	DWORD sz;
	CTime t = CTime::GetCurrentTime();
	sz=m_Log.GetLineCount();
	sz=m_Log.LineIndex(sz-1);
	m_Log.SetSel(sz,-1,FALSE);
	sz=wsprintf(Buf,"<%02i:%02i:%02i> ",t.GetHour(),t.GetMinute(),t.GetSecond());
	sz+=wsprintf(&Buf[sz],szString,Val1,Val2,Val3,Val4);
	wsprintf(&Buf[sz],"\r\n");
	m_Log.ReplaceSel(Buf);
}


static void Path_Link(char *Dst,char *Path,char *FName) {
	int i;
	strcpy(Dst,Path);
	i=strlen(Dst);
	if(i) {
		if(Dst[i-1] != '\\') {
			Dst[i]='\\';
			i++;
		}
	}
	strcpy(&Dst[i],FName);
}


static BOOL GetFileName(char *Dst, char *Path, char *FName) {
	Dst[0]=0;
	if(FName && FName[0]) {
		FILE *f;
		char name[MAX_PATH];
		char *ptr,*p;
		int Val;
		strcpy(name,FName);
		while(1) {
			Path_Link(Dst,Path,name);
			f=fopen(Dst,"rb");
			if(f == 0) {
				break;
			}
			fclose(f);
			ptr=name;
			ptr+=strlen(name)-1;
			Val=1;
			p=ptr+1;
			if(ptr[0] == ')') {
				while(p>name) {
					if(p[0] == '(') {
						Val=atoi(p+1);
						Val++;
						break;
					}
					p--;
				}
				if(p == name) p=ptr+1;
			}
			sprintf(p,"(%i)",Val);
		}
		return 1;
	}
	return 0;
}



static DEK_RESULT LoadData(CAtariReceiverDlg *obj, unsigned char *Buffer, DWORD *retSize)
{
	DWORD i,j,SmpSize;
	unsigned char *Smp;

	Smp=obj->m_rec.GetNextBlock(&SmpSize);
	if(Smp) {
		for(i=obj->m_vChannel,j=0;i<SmpSize;i+=2,j++) {
			Buffer[j]=Smp[i];
		}
		retSize[0]=j;
		return DEK_OK;
	} else {
		return DEK_OUT_OF_DATA;
	}
}

static BOOL ReadFName(CAtariReceiverDlg *obj,CAtariDekoder *m_dek,unsigned char *Buffer,char *FName)
{
	unsigned char B,suma,Load;
	DWORD pos,Size;
	DEK_RESULT res;

	obj->OUT0("Waiting for the header");
	Load=0;
	FName[0]=0;
	while(1) {
		if(0 != obj->m_StopThread) break;
		if(Load ==0) {
			suma=0;
			pos=0;
			FName[0]=0;
			m_dek->SetNextBlock();
			Load=1;
		}
		res=m_dek->GetNextByte(&B);
		if(res == DEK_OK) {
			switch(pos) {
			case 0:
				if(B != 0) {
					Load=0;
				}
				break;
			case 1:
				if(B != 0xFF) {
					Load=0;
				}
				break;
			default:
				FName[pos-2]=B;
			}
			if(pos == 12) {
				FName[pos-2]=0;
				if(suma == B) {
					return TRUE;
				} else {
					return FALSE;
				}
			}
			pos++;
			suma+=B;
		} else {
			if(res == DEK_IO_ERROR) {
				if(pos > 0) {
					return FALSE;
				}
				Load=0;
			}
			res=LoadData(obj,Buffer,&Size);
			if(res == DEK_OK) {
				m_dek->SetBuffer(Buffer,Size);
			} else {
				Sleep(50);
			}
		}
	}
	return FALSE;
}

#define MAX_BLOCK_SIZE	3072

static BOOL ReadExisting(CAtariReceiverDlg *obj,char *FName, unsigned char *Buffer, DWORD *fSize, DWORD Counter)
{
	char FilePath[MAX_PATH];
	char *ptr;
	long hd;
	FILE *f;
	BOOL ret=FALSE;

	if(obj->m_vUseDst) {
		_finddata_t dta;
		DWORD fileSize,reqSize;
		Path_Link(FilePath,(char*)((LPCTSTR)obj->m_DstPath),FName);
		ptr=FilePath+strlen(FilePath);
		sprintf(ptr,".ERROR*");
		hd=_findfirst(FilePath,&dta);
		if(hd != -1) {
			do {
				if(0 == (dta.attrib & _A_SUBDIR)) {
					Path_Link(FilePath,(char*)((LPCTSTR)obj->m_DstPath),dta.name);
					f=fopen(FilePath,"rb");
					if(f) {
						reqSize=(Counter-1)*MAX_BLOCK_SIZE;
						fseek(f,0,SEEK_END);
						fileSize=ftell(f);
						if(fileSize>reqSize) {
							fseek(f,reqSize,SEEK_SET);
							reqSize=fread(Buffer,1,MAX_BLOCK_SIZE,f);
							if(0 == ferror(f)) {
								fSize[0]=reqSize;
								ret=TRUE;
							} else {
								// read error - continue next file
							}
						}
						fclose(f);
						if(ret) break;
					}
				}
			} while(0 == _findnext(hd,&dta));
			_findclose(hd);
		}
		if(ret) {
			obj->OUT1("    Using data from file '%s'",dta.name);
		}
	}
	return ret;
}

static BOOL ReadAllFile(CAtariReceiverDlg *obj,CAtariDekoder *m_dek,unsigned char *Buffer,char *FName)
{
	unsigned char B,suma,Load;
	DWORD pos,Size,fSize,Counter;
	DEK_RESULT res;
	unsigned char Block[MAX_BLOCK_SIZE+1];
	char FilePath[MAX_PATH];
	char FilePath2[MAX_PATH];
	FILE *f;
	GetFileName(FilePath,(char*)((LPCTSTR)obj->m_DstPath),FName);
	f=fopen(FilePath,"wb");
	if(f) {
//		obj->OUT0("    Waiting for the first record");
		Load=0;
		Counter=1;
		while(1) {
			if(0 != obj->m_StopThread) break;
			if(Load ==0) {
				suma=0;
				pos=0;
				fSize=0;
				m_dek->SetNextBlock();
				Load=1;
			}
			res=m_dek->GetNextByte(&B);
			if(res == DEK_OK) {
				switch(pos) {
				case 0:
					fSize=B;
					break;
				case 1:
					Size=B;
					Size<<=8;
					fSize|=Size;
					break;
				default:
					Block[pos-2]=B;
				}
				if(fSize>MAX_BLOCK_SIZE) {
					obj->OUT2("    Record %i is too long (%i bytes)",Counter,fSize);
					if(ReadExisting(obj,FName,Block,&fSize,Counter)) {
						goto SAVE_FILE;
					} else {
						break;
					}
				}
				if(pos == (MAX_BLOCK_SIZE+2)) {
					if(suma == B) {
SAVE_FILE:
						if(fSize) {
							if(1 != fwrite(Block,fSize,1,f)) {
								obj->OUT0("    Saveing error");
								break;
							}
						}
						obj->OUT1("    Saved record %i",Counter);
						Counter++;
						Load=0;
						if(fSize<MAX_BLOCK_SIZE) {
							fclose(f);
							return TRUE;
						}
					} else {
						obj->OUT1("    Checksum error in record %i",Counter);
						if(ReadExisting(obj,FName,Block,&fSize,Counter)) {
							goto SAVE_FILE;
						} else {
							break;
						}
					}
				}
				pos++;
				suma+=B;
			} else {
				if(res == DEK_IO_ERROR) {
					if(pos > 0) {
						obj->OUT2("    Signal is corrupted in record %i after byte %i",Counter,pos);
						if(ReadExisting(obj,FName,Block,&fSize,Counter)) {
							goto SAVE_FILE;
						} else {
							break;
						}
					} else {
						Load=0;
//						obj->OUT0("    Loader corrupted, trying again");
					}
				} else {
					res=LoadData(obj,Buffer,&Size);
					if(res == DEK_OK) {
						m_dek->SetBuffer(Buffer,Size);
					} else {
						Sleep(50);
					}
				}
			}
		}
		fclose(f);
		if(Counter == 1) {
			DeleteFile(FilePath);
		} else {
			wsprintf((char*)Block,"%s.ERROR",FName);
			GetFileName(FilePath2,(char*)((LPCTSTR)obj->m_DstPath),(char*)Block);
			MoveFile(FilePath,FilePath2);
		}
	} else obj->OUT1("    Can't open file to save '%s'",FName);
	return FALSE;
}



static DWORD WINAPI ThreadProc(LPVOID lpParameter) {
	CAtariReceiverDlg *obj=(CAtariReceiverDlg*)lpParameter;
	CAtariDekoder *m_dek;
	unsigned char *Buffer;
	DWORD Size;
	char Header[16];

	Size=obj->m_rec.GetBufferSize();
	Buffer=(unsigned char *)GlobalAlloc(GMEM_FIXED,Size);
	if(Buffer) {
		// Selected device
		switch(obj->m_vDevice) {
		case 0:
			m_dek=new CDekoderXL;
			break;
		case 1:
			m_dek=new CDekoderFM2XL;
			break;
		default:
			break;
		}
		if(m_dek) {
			while(1) {
				if(0 != obj->m_StopThread) {
					break;
				}
				if(ReadFName(obj,m_dek,Buffer,Header)) {
					for(Size=9;Header[Size] == ' ';Size--) {
					}
					Header[Size+1]=0;
					for(Size=0;Header[Size];Size++) {
						if(Header[Size] < ' ' || Header[Size] > 'z') {
							Header[Size]='_';
						}
					}
					obj->OUT1("Found : '%s'",Header);
					if(ReadAllFile(obj,m_dek,Buffer,Header)) {
						obj->OUT1("    File saved : '%s'",Header);
					} else {
						obj->OUT1("    Error reading : '%s'",Header);
					}
				} else {
					//if(0 == obj->m_StopThread) obj->OUT0("Signal corrupted");
				}
			}
			delete m_dek;
		} else obj->OUT0("ERROR - Driver not implemented yet");
		GlobalFree(Buffer);
	} else obj->OUT1("ERROR - Can't alloc memory (%i bytes)",Size);
	obj->m_Mode=WAITING;
	obj->UpdateMode();
	obj->m_StopThread=0;
	return 0;
}


void CAtariReceiverDlg::StartThread()
{
	HANDLE hd;
	DWORD dw;
	SECURITY_ATTRIBUTES attr;
	if(m_StopThread == 0) {
//		hd=GetCurrentProcess();
//		SetPriorityClass(hd,HIGH_PRIORITY_CLASS);
	//	m_StopThread=0;
		attr.bInheritHandle = TRUE;
		attr.nLength = sizeof(attr);
		attr.lpSecurityDescriptor = 0;
		hd=CreateThread(&attr,0,ThreadProc,this,0,&dw);
		if(hd) {
			::SetThreadPriority(hd,THREAD_PRIORITY_TIME_CRITICAL);
			CloseHandle(hd);
		}
	}
}

void CAtariReceiverDlg::StopThread()
{
	if(m_StopThread == 0) {
		m_StopThread=1;
	}
//	HANDLE hd=GetCurrentProcess();
//	SetPriorityClass(hd,NORMAL_PRIORITY_CLASS);
}

void CAtariReceiverDlg::OnButtonFileDestination() 
{
	// TODO: Add your control notification handler code here
	BROWSEINFO bi;
	LPMALLOC pMalloc; 
	LPITEMIDLIST pidl;
	char szPath[MAX_PATH];

	ZeroMemory(&bi, sizeof(bi));
	bi.hwndOwner = m_hWnd;
	bi.pszDisplayName = "Display Name"; 
	bi.lpszTitle = TEXT("Select folder"); 
	bi.ulFlags = BIF_DONTGOBELOWDOMAIN | BIF_RETURNONLYFSDIRS; 
 
	pidl = SHBrowseForFolder(&bi); 
	if(pidl) {
		SHGetPathFromIDList(pidl, szPath);
		if (SHGetMalloc(&pMalloc) == NOERROR) { 
			pMalloc->Free((void *)pidl); 
		}
		m_DstPath=szPath;
	}
	UpdateMode();
}


#define REG_KEY		"SOFTWARE\\HenrykCygertApps\\AtariTurbo\\1.01"

void CAtariReceiverDlg::SaveConfig()
{
	HKEY key;
	if(ERROR_SUCCESS == RegCreateKey(HKEY_LOCAL_MACHINE,REG_KEY,&key)) {
		RegSetValueEx(key,"Destination",0,REG_SZ,(BYTE*)((LPCTSTR)m_DstPath),m_DstPath.GetLength());
		RegSetValueEx(key,"Device",0,REG_DWORD,(BYTE*)&m_vDevice,4);
		RegSetValueEx(key,"Channel",0,REG_DWORD,(BYTE*)&m_vChannel,4);
		RegSetValueEx(key,"UseExistingFiles",0,REG_DWORD,(BYTE*)&m_vUseDst,4);
	}
}

void CAtariReceiverDlg::LoadConfig()
{
	HKEY key;
	char Buf[MAX_PATH];
	DWORD Size;
	DWORD Type;
	if(ERROR_SUCCESS == RegOpenKey(HKEY_LOCAL_MACHINE,REG_KEY,&key)) {
		Size=sizeof(Buf);
		RegQueryValueEx(key,"Destination",0,&Type,(BYTE*)Buf,&Size);
		m_DstPath=Buf;

		Size=4;
		RegQueryValueEx(key,"Device",0,&Type,(BYTE*)&m_vDevice,&Size);
		Size=4;
		RegQueryValueEx(key,"Channel",0,&Type,(BYTE*)&m_vChannel,&Size);
		Size=4;
		RegQueryValueEx(key,"UseExistingFiles",0,&Type,(BYTE*)&m_vUseDst,&Size);
	}
}

void CAtariReceiverDlg::OnButtonClearLog() 
{
	// TODO: Add your control notification handler code here
	m_Log.SetWindowText("");
}

void CAtariReceiverDlg::OnButtonAbout() 
{
	// TODO: Add your control notification handler code here
	CAbout dlg;
	dlg.DoModal();
}
